@@ -71,6 +71,9 @@ void		 up_device_sibling_discovered	(UpDevice	*device,
 						 GObject	*sibling);
 gboolean	 up_device_refresh_internal	(UpDevice	*device,
 						 UpRefreshReason reason);
+void		 up_device_unregister		(UpDevice	*device);
+gboolean	 up_device_register		(UpDevice	*device);
+gboolean	 up_device_is_registered	(UpDevice	*device);
 
 G_END_DECLS
 
