@@ -25,6 +25,7 @@
 #include <glib/gstdio.h>
 #include <up-history-item.h>
 #include <stdlib.h>
+#include <unistd.h>
 #include <errno.h>
 #include "up-backend.h"
 #include "up-daemon.h"
@@ -32,12 +33,11 @@
 #include "up-device-list.h"
 #include "up-history.h"
 #include "up-native.h"
-#include "up-polkit.h"
-#include "up-qos.h"
-#include "up-wakeups.h"
 
 gchar *history_dir = NULL;
 
+#define DBUS_SYSTEM_SOCKET "/var/run/dbus/system_bus_socket"
+
 static void
 up_test_native_func (void)
 {
@@ -76,7 +76,7 @@ up_test_device_func (void)
 {
 	UpDevice *device;
 
-	device = up_device_new ();
+	device = up_device_new (NULL, NULL);
 	g_assert (device != NULL);
 
 	/* unref */
@@ -88,7 +88,7 @@ up_test_device_list_func (void)
 {
 	UpDeviceList *list;
 	GObject *native;
-	GObject *device;
+	UpDevice *device;
 	GObject *found;
 	gboolean ret;
 
@@ -97,8 +97,8 @@ up_test_device_list_func (void)
 
 	/* add device */
 	native = g_object_new (G_TYPE_OBJECT, NULL);
-	device = g_object_new (G_TYPE_OBJECT, NULL);
-	ret = up_device_list_insert (list, native, device);
+	device = up_device_new (NULL, native);
+	ret = up_device_list_insert (list, device);
 	g_assert (ret);
 
 	/* find device */
@@ -141,7 +141,7 @@ up_test_history_func (void)
 	gboolean ret;
 	GPtrArray *array;
 	gchar *filename;
-	UpHistoryItem *item;
+	UpHistoryItem *item, *item2, *item3;
 
 	history = up_history_new ();
 	g_assert (history != NULL);
@@ -163,17 +163,23 @@ up_test_history_func (void)
 	array = up_history_get_data (history, UP_HISTORY_TYPE_CHARGE, 10, 100);
 	g_assert (array != NULL);
 	g_assert_cmpint (array->len, ==, 0);
+	g_ptr_array_unref (array);
 
-	/* setup some fake device */
+	/* setup some fake device and three data points */
 	up_history_set_state (history, UP_DEVICE_STATE_CHARGING);
+	up_history_set_charge_data (history, 85);
+	up_history_set_rate_data (history, 0.99f);
+	up_history_set_time_empty_data (history, 12346);
+	up_history_set_time_full_data (history, 54322);
+
+	g_usleep (2 * G_USEC_PER_SEC);
 	up_history_set_charge_data (history, 90);
 	up_history_set_rate_data (history, 1.00f);
 	up_history_set_time_empty_data (history, 12345);
 	up_history_set_time_full_data (history, 54321);
 
-	/* sleep for a little bit */
-	g_usleep (3 * G_USEC_PER_SEC);
-	up_history_set_charge_data (history, 91);
+	g_usleep (2 * G_USEC_PER_SEC);
+	up_history_set_charge_data (history, 95);
 	up_history_set_rate_data (history, 1.01f);
 	up_history_set_time_empty_data (history, 12344);
 	up_history_set_time_full_data (history, 54320);
@@ -181,13 +187,44 @@ up_test_history_func (void)
 	/* get data for last 10 seconds */
 	array = up_history_get_data (history, UP_HISTORY_TYPE_CHARGE, 10, 100);
 	g_assert (array != NULL);
-	g_assert_cmpint (array->len, ==, 2);
+	g_assert_cmpint (array->len, ==, 3);
 
 	/* get the first item, which should be the most recent */
 	item = g_ptr_array_index (array, 0);
 	g_assert (item != NULL);
-	g_assert_cmpint (up_history_item_get_value (item), ==, 91);
+	g_assert_cmpint (up_history_item_get_value (item), ==, 95);
 	g_assert_cmpint (up_history_item_get_time (item), >, 1000000);
+
+        /* the second one ought to be older */
+	item2 = g_ptr_array_index (array, 1);
+	g_assert (item2 != NULL);
+	g_assert_cmpint (up_history_item_get_value (item2), ==, 90);
+	g_assert_cmpint (up_history_item_get_time (item2), <, up_history_item_get_time (item));
+
+        /* third one is the oldest */
+	item3 = g_ptr_array_index (array, 2);
+	g_assert (item3 != NULL);
+	g_assert_cmpint (up_history_item_get_value (item3), ==, 85);
+	g_assert_cmpint (up_history_item_get_time (item3), <, up_history_item_get_time (item2));
+
+	g_ptr_array_unref (array);
+
+        /* request fewer items than we have in our history; should have the
+         * same order: first one is the most recent, and the data gets
+         * interpolated */
+	array = up_history_get_data (history, UP_HISTORY_TYPE_CHARGE, 10, 2);
+	g_assert (array != NULL);
+	g_assert_cmpint (array->len, ==, 2);
+
+	item = g_ptr_array_index (array, 0);
+	g_assert (item != NULL);
+	item2 = g_ptr_array_index (array, 1);
+	g_assert (item2 != NULL);
+
+	g_assert_cmpint (up_history_item_get_time (item), >, 1000000);
+	g_assert_cmpint (up_history_item_get_value (item), ==, 95);
+	g_assert_cmpint (up_history_item_get_value (item2), ==, 87);
+
 	g_ptr_array_unref (array);
 
 	/* force a save to disk */
@@ -208,10 +245,10 @@ up_test_history_func (void)
 	/* get data for last 10 seconds */
 	array = up_history_get_data (history, UP_HISTORY_TYPE_CHARGE, 10, 100);
 	g_assert (array != NULL);
-	g_assert_cmpint (array->len, ==, 3); /* we have inserted an unknown as the first entry */
+	g_assert_cmpint (array->len, ==, 4); /* we have inserted an unknown as the first entry */
 	item = g_ptr_array_index (array, 1);
 	g_assert (item != NULL);
-	g_assert_cmpint (up_history_item_get_value (item), ==, 91);
+	g_assert_cmpint (up_history_item_get_value (item), ==, 95);
 	g_assert_cmpint (up_history_item_get_time (item), >, 1000000);
 	g_ptr_array_unref (array);
 
@@ -237,53 +274,15 @@ up_test_history_func (void)
 	rmdir (history_dir);
 }
 
-static void
-up_test_polkit_func (void)
-{
-	UpPolkit *polkit;
-
-	polkit = up_polkit_new ();
-	g_assert (polkit != NULL);
-
-	/* unref */
-	g_object_unref (polkit);
-}
-
-static void
-up_test_qos_func (void)
-{
-	UpQos *qos;
-
-	qos = up_qos_new ();
-	g_assert (qos != NULL);
-
-	/* unref */
-	g_object_unref (qos);
-}
-
-static void
-up_test_wakeups_func (void)
-{
-	UpWakeups *wakeups;
-
-	wakeups = up_wakeups_new ();
-	g_assert (wakeups != NULL);
-
-	/* unref */
-	g_object_unref (wakeups);
-}
-
 int
 main (int argc, char **argv)
 {
+#if !defined(GLIB_VERSION_2_36)
 	g_type_init ();
+#endif
 	g_test_init (&argc, &argv, NULL);
 
-	/* make check, vs. make distcheck */
-	if (g_file_test ("../etc/UPower.conf", G_FILE_TEST_EXISTS))
-		g_setenv ("UPOWER_CONF_FILE_NAME", "../etc/UPower.conf", TRUE);
-	else
-		g_setenv ("UPOWER_CONF_FILE_NAME", "../../etc/UPower.conf", TRUE);
+	g_setenv ("UPOWER_CONF_FILE_NAME", UPOWER_CONF_PATH, TRUE);
 
 	/* tests go here */
 	g_test_add_func ("/power/backend", up_test_backend_func);
@@ -291,9 +290,6 @@ main (int argc, char **argv)
 	g_test_add_func ("/power/device_list", up_test_device_list_func);
 	g_test_add_func ("/power/history", up_test_history_func);
 	g_test_add_func ("/power/native", up_test_native_func);
-	g_test_add_func ("/power/polkit", up_test_polkit_func);
-	g_test_add_func ("/power/qos", up_test_qos_func);
-	g_test_add_func ("/power/wakeups", up_test_wakeups_func);
 	g_test_add_func ("/power/daemon", up_test_daemon_func);
 
 	return g_test_run ();
